<?php
/**************************************************************************\
* eGroupWare                                                               *
* http://www.egroupware.org                                                *
* --------------------------------------------                             *
*  This program is free software; you can redistribute it and/or modify it *
*  under the terms of the GNU General Public License as published by the   *
*  Free Software Foundation; either version 2 of the License, or (at your  *
*  option) any later version.                                              *
\**************************************************************************/

/**
 * Classe base dos Jobs
 * @author Sidnei Augusto Drovetto Junior - drovetto@gmail.com
 * @version 1.0
 * @abstract
 * @license http://www.gnu.org/copyleft/gpl.html GPL
 * @package Workflow
 * @subpackage Job
 */
abstract class JobBase
{
	/**
	 * @var int $jobID O ID do Job
	 * @access private
	 */
	private $jobID;

	/**
	 * @var int $processID O ID do Processo
	 * @access private
	 */
	private $processID;

	/**
	 * @var string $currentDate Uma string cujo contedo  um data e horrio devidamente formatados
	 * @access private
	 */
	private $currentDate;

	/**
	 * @var object $jobManager Objeto da classe JobManager
	 * @access private
	 */
	private $jobManager;

	/**
	 * @var bool $userLog Indica se o usurio gravou algum log (true) ou no (false)
	 * @access private
	 */
	private $userLog;

	/**
	 * @var float $maximumExecutionTime O tempo mximo de execuo do Job
	 * @access private
	 */
	private $maximumExecutionTime;

	/**
	 * @var float $timeStart Incio do Job (em microsegundos)
	 * @access private
	 */
	private $timeStart;

	/**
	 * @var bool $testMode Indica se o job est sendo executado em modo de teste (true) ou no (false)
	 * @access protected
	 */
	protected $testMode;

	/**
	 * @var array $environment Contm algumas variveis/objetos relevantes para a execuo de alguns Jobs
	 * @access protected
	 */
	protected $environment;

	/**
	 * Construtor da classe JobBase
	 * @param int $jobID O ID do Job que ser atualizado
	 * @param int $processID O ID do processo a qual o Job pertence
	 * @param string $currentDate Uma string cujo contedo  um data e horrio devidamente formatados
	 * @param bool $testMode Indica se o job est sendo executado em modo de teste (true) ou no (false)
	 * @param float $maximumExecutionTime O tempo mximo de execuo do Job (em segundos)
	 * @return object Objeto da classe JobBase
	 * @final
	 * @access public
	 */
	final public function JobBase($jobID, $processID, $currentDate, $testMode, $maximumExecutionTime)
	{
		require_once 'common.inc.php';
		$GLOBALS['workflow']['factory']->getInstance('WorkflowMacro')->prepareEnvironment();
		$this->jobID = $jobID;
		$this->processID = $processID;
		$this->currentDate = $currentDate;
		$this->jobManager = &$GLOBALS['workflow']['factory']->getInstance('WorkflowJobManager');
		$this->testMode = $testMode;
		$this->maximumExecutionTime = $maximumExecutionTime;
		$this->timeStart = microtime(true);
		$this->userLog = false;

		$this->prepareEnvironment();
	}

	/**
	 * Indica que a execuo do Job falhou
	 * @param string $message A mensagem que ser armazenada
	 * @return void
	 * @final
	 * @access public
	 */
	final public function fail($message)
	{
		$this->jobManager->writeLog($this->jobID, $this->currentDate, $message, JobManager::STATUS_JOB_FAIL);
		$this->userLog = true;
	}

	/**
	 * Indica que a execuo do Job foi bem sucedida
	 * @param string $message A mensagem que ser armazenada
	 * @return void
	 * @final
	 * @access public
	 */
	final public function success($message)
	{
		$this->jobManager->writeLog($this->jobID, $this->currentDate, $message, JobManager::STATUS_JOB_SUCCESS);
		$this->userLog = true;
	}

	/**
	 * Grava uma mensagem de log padro (caso o desenvolvedor no chame os mtodos "fail" ou "success")
	 * @return void
	 * @final
	 * @access private
	 */
	final private function defaultLog()
	{
		$this->jobManager->writeLog($this->jobID, $this->currentDate, 'Job executado', JobManager::STATUS_UNKNOWN);
		$this->userLog = true;
	}

	/**
	 * Prepara o ambiente para a execuo do Job
	 * @return void
	 * @final
	 * @access private
	 */
	final private function prepareEnvironment()
	{
		$GLOBALS['workflow']['factory']->getInstance('WorkflowMacro')->prepareProcessEnvironment($this->processID);

		require_once PHPGW_SERVER_ROOT . '/workflow/inc/class.process_factory.inc.php';

		$this->environment = array();
		$this->environment['factory']  = &new ProcessFactory();
	}

	/**
	 * Procedimentos que so executados na finalizao do Job
	 * @return void
	 * @final
	 * @access private
	 */
	final public function finalize()
	{
		if (!$this->userLog)
			$this->defaultLog();
	}

	/**
	 * Pega o tempo de execuo do Job (at o momento)
	 * @return float O tempo de execuo do Job
	 * @final
	 * @access public
	 */
	final public function getExecutionTime()
	{
		return microtime(true) - $this->timeStart;
	}

	/**
	 * Pega o tempo mximo de execuo do Job
	 * @return float O tempo mximo de execuo do Job
	 * @final
	 * @access public
	 */
	final public function getMaximumExectuionTime()
	{
		return $this->maximumExecutionTime;
	}

	/**
	 *  o mtodo que contm o cdigo do Job que ser executado
	 * @return void
	 * @abstract
	 * @access public
	 */
	abstract public function run();
}
?>
